﻿using System;
using Windows.System;
using Windows.UI.Input;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

namespace Chapter6
{
    public sealed partial class MainPage : Page
    {
        GestureRecognizer gestureRecognizer;

        public MainPage()
        {
            this.InitializeComponent();

            // Initialize gesture recognizer
            gestureRecognizer = new GestureRecognizer();
            gestureRecognizer.GestureSettings = GestureSettings.Tap |
              GestureSettings.RightTap | GestureSettings.DoubleTap |
              GestureSettings.HoldWithMouse | GestureSettings.CrossSlide;

            // Attach handlers for recognized gestures (not shown in this listing)
            gestureRecognizer.Tapped += GestureRecognizer_Tapped;
            gestureRecognizer.RightTapped += GestureRecognizer_RightTapped;
            gestureRecognizer.Holding += GestureRecognizer_Holding;
            gestureRecognizer.CrossSliding += GestureRecognizer_CrossSliding;
        }

        //
        // Handlers for recognized gestures:
        //

        async void GestureRecognizer_CrossSliding(GestureRecognizer sender, CrossSlidingEventArgs args)
        {
            Windows.UI.Popups.MessageDialog d = new Windows.UI.Popups.MessageDialog("CrossSliding");
            await d.ShowAsync();
        }

        async void GestureRecognizer_Holding(GestureRecognizer sender, HoldingEventArgs args)
        {
            Windows.UI.Popups.MessageDialog d = new Windows.UI.Popups.MessageDialog("Holding");
            await d.ShowAsync();
        }

        async void GestureRecognizer_RightTapped(GestureRecognizer sender, RightTappedEventArgs args)
        {
            Windows.UI.Popups.MessageDialog d = new Windows.UI.Popups.MessageDialog("RightTapped");
            await d.ShowAsync();
        }

        async void GestureRecognizer_Tapped(GestureRecognizer sender, TappedEventArgs args)
        {
            Windows.UI.Popups.MessageDialog d = new Windows.UI.Popups.MessageDialog("Tapped");
            await d.ShowAsync();
        }

        //
        // Forward all pointer input to the gesture recognizer so it can do its job:
        //

        protected override void OnPointerPressed(PointerRoutedEventArgs e)
        {
            CapturePointer(e.Pointer);
            gestureRecognizer.ProcessDownEvent(e.GetCurrentPoint(this));
        }

        protected override void OnPointerMoved(PointerRoutedEventArgs e)
        {
            // Pass all intermediate potentially-coalesced points:
            gestureRecognizer.ProcessMoveEvents(e.GetIntermediatePoints(this));
        }

        protected override void OnPointerReleased(PointerRoutedEventArgs e)
        {
            gestureRecognizer.ProcessUpEvent(e.GetCurrentPoint(this));
            ReleasePointerCapture(e.Pointer);
        }

        protected override void OnPointerCanceled(PointerRoutedEventArgs e)
        {
            OnPointerReleased(e);
        }

        protected override void OnPointerCaptureLost(PointerRoutedEventArgs e)
        {
            OnPointerReleased(e);
        }

        // This is for mouse only, but here for completeness:
        protected override void OnPointerWheelChanged(PointerRoutedEventArgs e)
        {
            bool isShiftKeyDown = ((e.KeyModifiers & VirtualKeyModifiers.Shift) ==
                                   VirtualKeyModifiers.Shift);
            bool isControlKeyDown = ((e.KeyModifiers & VirtualKeyModifiers.Control) ==
                                     VirtualKeyModifiers.Control);

            gestureRecognizer.ProcessMouseWheelEvent(e.GetCurrentPoint(this),
              isShiftKeyDown, isControlKeyDown);
        }
    }
}