﻿using System;
using System.Collections.Concurrent;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace MultipleWindows
{
  /// <summary>
  /// Provides application-specific behavior to supplement the default Application class.
  /// </summary>
  sealed partial class App : Application
  {
    // The pending tasks created by CreateWindowAsync
    ConcurrentQueue<TaskCompletionSource<Window>> taskWrappers = new ConcurrentQueue<TaskCompletionSource<Window>>();

    /// <summary>
    /// Initializes the singleton application object.  This is the first line of authored code
    /// executed, and as such is the logical equivalent of main() or WinMain().
    /// </summary>
    public App()
    {
      this.InitializeComponent();
      this.Suspending += OnSuspending;
    }

    // Create a new window.
    // This wrapper method enables awaiting.
    public Task<Window> CreateWindowAsync()
    {
      // Create a Task that the caller can await
      TaskCompletionSource<Window> taskWrapper = new TaskCompletionSource<Window>();
      this.taskWrappers.Enqueue(taskWrapper);

      // Create the secondary view, which calls Application.OnWindowCreated
      // on its own UI thread
      CoreApplication.CreateNewView();

      // Return the Task
      return taskWrapper.Task;
    }

    protected override void OnWindowCreated(WindowCreatedEventArgs args)
    {
      CoreApplicationView view = CoreApplication.GetCurrentView();
      if (!view.IsMain)
      {
        // This is a secondary view, so mark the in-progress Task as complete
        TaskCompletionSource<Window> taskWrapper;
        if (taskWrappers.TryDequeue(out taskWrapper))
          taskWrapper.SetResult(args.Window);
        else
          throw new InvalidOperationException();
      }
    }

    /// <summary>
    /// Invoked when the application is launched normally by the end user.  Other entry points
    /// will be used such as when the application is launched to open a specific file.
    /// </summary>
    /// <param name="e">Details about the launch request and process.</param>
    protected override void OnLaunched(LaunchActivatedEventArgs e)
    {

#if DEBUG
      if (System.Diagnostics.Debugger.IsAttached)
      {
        this.DebugSettings.EnableFrameRateCounter = true;
      }
#endif

      Frame rootFrame = Window.Current.Content as Frame;

      // Do not repeat app initialization when the Window already has content,
      // just ensure that the window is active
      if (rootFrame == null)
      {
        // Create a Frame to act as the navigation context and navigate to the first page
        rootFrame = new Frame();

        if (e.PreviousExecutionState == ApplicationExecutionState.Terminated)
        {
          //TODO: Load state from previously suspended application
        }

        // Place the frame in the current Window
        Window.Current.Content = rootFrame;
      }

      if (rootFrame.Content == null)
      {
        // When the navigation stack isn't restored navigate to the first page,
        // configuring the new page by passing required information as a navigation
        // parameter
        if (!rootFrame.Navigate(typeof(MainPage), e.Arguments))
        {
          throw new Exception("Failed to create initial page");
        }
      }
      // Ensure the current window is active
      Window.Current.Activate();
    }

    /// <summary>
    /// Invoked when application execution is being suspended.  Application state is saved
    /// without knowing whether the application will be terminated or resumed with the contents
    /// of memory still intact.
    /// </summary>
    /// <param name="sender">The source of the suspend request.</param>
    /// <param name="e">Details about the suspend request.</param>
    private void OnSuspending(object sender, SuspendingEventArgs e)
    {
      var deferral = e.SuspendingOperation.GetDeferral();
      //TODO: Save application state and stop any background activity
      deferral.Complete();
    }
  }
}
