﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Data.Pdf;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

namespace PdfExample
{
  public sealed partial class MainPage : Page
  {
    PdfDocument document;
    PdfPage page;

    public MainPage()
    {
      this.InitializeComponent();
      ShowUserSelectedFile();
    }

    async Task ShowUserSelectedFile()
    {
      // Get a PDF file from the user
      FileOpenPicker picker = new FileOpenPicker();
      picker.FileTypeFilter.Add(".pdf");
      StorageFile file = await picker.PickSingleFileAsync();
      if (file != null)
      {
        // Throws an exception if the document is password-protected:
        document = await PdfDocument.LoadFromFileAsync(file);

        // Get the first page (You can see how many exist via document.PageCount)
        page = document.GetPage(0);
        {
          // Render the image to an in-memory stream then display it
          using (InMemoryRandomAccessStream ras = new InMemoryRandomAccessStream())
          {
            await page.RenderToStreamAsync(ras);

            // Load the data into a BitmapImage
            BitmapImage source = new BitmapImage();
            source.SetSource(ras);

            // Assign the BitmapImage to the Image element on the page
            image.Source = source;
          }
        }
      }
    }

    async void ScrollViewer_ViewChanged(object sender, ScrollViewerViewChangedEventArgs e)
    {
      if (!e.IsIntermediate && this.page != null)
      {
        // Replace the current zoomed image content with one optimized for the current zoom factor
        using (InMemoryRandomAccessStream ras = new InMemoryRandomAccessStream())
        {
          uint newHeight = (uint)((sender as ScrollViewer).ZoomFactor * this.page.Size.Height);

          await this.page.RenderToStreamAsync(ras, new PdfPageRenderOptions { DestinationHeight = newHeight });

          // Load the data into a BitmapImage
          BitmapImage source = new BitmapImage();
          source.SetSource(ras);

          // Assign the BitmapImage to the Image element on the page
          image.Source = source;
        }
      }
    }
  }
}
