﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Media.MediaProperties;
using Windows.Media.Transcoding;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace ConvertMP4toWAV
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            ConvertUserSelectedMp4ToWmv();
        }

        async Task ConvertUserSelectedMp4ToWmv()
        {
            MessageDialog errorDialog = null;

            // Get a source MP4 file from the user
            FileOpenPicker openPicker = new FileOpenPicker();
            openPicker.FileTypeFilter.Add(".mp4");
            StorageFile sourceFile = await openPicker.PickSingleFileAsync();
            if (sourceFile != null)
            {
                // Get a target WMV file from the user
                FileSavePicker savePicker = new FileSavePicker();
                savePicker.FileTypeChoices.Add("WMV file", new string[] { ".wmv" });
                StorageFile targetFile = await savePicker.PickSaveFileAsync();
                if (targetFile != null)
                {
                    try
                    {
                        // Specify the output format
                        MediaEncodingProfile profile =
                          MediaEncodingProfile.CreateWmv(VideoEncodingQuality.Auto);

                        // Transcode!
                        MediaTranscoder transcoder = new MediaTranscoder();
                        PrepareTranscodeResult result = await transcoder.PrepareFileTranscodeAsync(
                                                                sourceFile, targetFile, profile);
                        if (result.CanTranscode)
                        {
                            await result.TranscodeAsync();
                        }
                        else
                        {
                            // Check result.FailureReason: InvalidProfile, CodecNotFound, or Unknown
                            errorDialog = new MessageDialog(result.FailureReason.ToString());
                        }
                    }
                    catch (Exception ex)
                    {
                        errorDialog = new MessageDialog(ex.ToString());
                    }
                }
            }

            if (errorDialog != null)
            {
                await errorDialog.ShowAsync();
            }
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }
}
