﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Media;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace BackgroundAudio
{
  /// <summary>
  /// An empty page that can be used on its own or navigated to within a Frame.
  /// </summary>
  public sealed partial class MainPage : Page
  {
    IRandomAccessStream stream;

    public MainPage()
    {
      this.InitializeComponent();

      // Enable the system-wide Play, Pause, and Stop buttons
      SystemMediaTransportControls mediaControls =
        SystemMediaTransportControls.GetForCurrentView();
      mediaControls.IsPlayEnabled = true;
      mediaControls.IsPauseEnabled = true;
      mediaControls.IsStopEnabled = true;

      // Provide a thumbnail
      mediaControls.DisplayUpdater.Thumbnail = RandomAccessStreamReference.CreateFromUri(new Uri("ms-appx:///Assets/Logo.png"));

      // This must be set before setting the music-specific properties
      mediaControls.DisplayUpdater.Type = MediaPlaybackType.Music;

      // Now we can set the music-specific properties
      mediaControls.DisplayUpdater.MusicProperties.Title = "The Title";
      mediaControls.DisplayUpdater.MusicProperties.Artist = "The Artist";

      // Commit the update
      mediaControls.DisplayUpdater.Update();

      // Update the MediaElement state whenever the system-wide state changes
      this.mediaElement.CurrentStateChanged += MediaElement_CurrentStateChanged;

      // Update the system-wide state whenever the MediaElement state changes
      mediaControls.ButtonPressed += SystemMediaTransportControls_ButtonPressed;

      PlayUserSelectedMusic();
    }

// Update the media transport controls to match the new MediaElement state
void MediaElement_CurrentStateChanged(object sender, RoutedEventArgs e)
{
  SystemMediaTransportControls mediaControls = SystemMediaTransportControls.GetForCurrentView();

  switch (this.mediaElement.CurrentState)
  {
    case MediaElementState.Playing:
      mediaControls.PlaybackStatus = MediaPlaybackStatus.Playing;
      break;
    case MediaElementState.Paused:
      mediaControls.PlaybackStatus = MediaPlaybackStatus.Paused;
      break;
    case MediaElementState.Stopped:
      mediaControls.PlaybackStatus = MediaPlaybackStatus.Stopped;
      break;
    case MediaElementState.Closed:
      mediaControls.PlaybackStatus = MediaPlaybackStatus.Closed;
      break;
    default:
      // Nothing to do for this state
      break;
  }
}

// Update the MediElement to match the new media transport controls state
async void SystemMediaTransportControls_ButtonPressed(SystemMediaTransportControls sender, SystemMediaTransportControlsButtonPressedEventArgs args)
{
  // This is invoked on a background thread
  await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
  {
    switch (args.Button)
    {
      // We only need to worry about the buttons we explicitly enabled
      case SystemMediaTransportControlsButton.Play:
        this.mediaElement.Play();
        break;
      case SystemMediaTransportControlsButton.Pause:
        this.mediaElement.Pause();
        break;
      case SystemMediaTransportControlsButton.Stop:
        this.mediaElement.Stop();
        break;
      default:
        // Unexpected button
        break;
    }
  });
}

    async Task PlayUserSelectedMusic()
    {
      // Get an MP3 file from the user
      FileOpenPicker picker = new FileOpenPicker();
      picker.FileTypeFilter.Add(".mp3");
      picker.SuggestedStartLocation = PickerLocationId.MusicLibrary;
      StorageFile file = await picker.PickSingleFileAsync();
      if (file != null)
      {
        this.stream = await file.OpenAsync(FileAccessMode.Read);
        this.mediaElement.SetSource(this.stream, "audio/mp3");
        this.mediaElement.AudioCategory = AudioCategory.BackgroundCapableMedia;
      }
    }

    void MediaElement_MediaOpened(object sender, RoutedEventArgs e)
    {
      this.stream.Dispose();
    }

    /// <summary>
    /// Invoked when this page is about to be displayed in a Frame.
    /// </summary>
    /// <param name="e">Event data that describes how this page was reached.  The Parameter
    /// property is typically used to configure the page.</param>
    protected override void OnNavigatedTo(NavigationEventArgs e)
    {
    }
  }
}
