﻿using System;
using System.Threading.Tasks;
using Windows.Devices.Geolocation;
using Windows.Devices.Geolocation.Geofencing;
using Windows.UI.Popups;
using Windows.UI.Xaml.Controls;

namespace Location
{
  public sealed partial class MainPage : Page
  {
    public MainPage()
    {
      InitializeComponent();

      // Get notified when a geofence is triggered
      GeofenceMonitor.Current.GeofenceStateChanged += OnGeofenceTriggered;

      CreateGeofence();
    }

    async Task CreateGeofence()
    {
      try
      {
        // Ensure that location is enabled, and get a starting point
        Geolocator locator = new Geolocator();
        Geoposition position = await locator.GetGeopositionAsync();

        // Define a 5-meter radius around the current location
        Geocircle circle = new Geocircle(position.Coordinate.Point.Position, 5);

        // Create a corresponding geofence that monitors any exit and enter with a 10-second dwell time
        Geofence fence = new Geofence("myFence", circle);

        try
        {
          // Add the geofence to the system-managed list for this app.
          // This addition persists across app restarts.
          GeofenceMonitor.Current.Geofences.Add(fence);
        }
        catch
        {
          ShowError("A geofence with the same ID is already in the list.");
        }

      }
      catch (UnauthorizedAccessException)
      {
        ShowError("Location is not enabled for this app.");
      }
    }

    // This handler is called on a background thread
    void OnGeofenceTriggered(GeofenceMonitor sender, object args)
    {
      foreach (GeofenceStateChangeReport report in GeofenceMonitor.Current.ReadReports())
      {
        switch (report.NewState)
        {
          case GeofenceState.Entered:
            // The fence has been entered!
            break;
          case GeofenceState.Exited:
            // The fence has been left!
            break;
          case GeofenceState.Removed:
            // We didn't ask to be notified for this condition
            break;
        }
      }
    }

    async Task ShowError(string message)
    {
      MessageDialog dialog = new MessageDialog(message);
      await dialog.ShowAsync();
    }
  }
}