﻿using System;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace CustomPanels
{
  public class SimpleCanvas : Panel
  {
    public static readonly DependencyProperty LeftProperty =
      DependencyProperty.RegisterAttached("Left", typeof(double),
      typeof(SimpleCanvas), new PropertyMetadata(0.0, OnPropertyChanged));

    public static readonly DependencyProperty TopProperty =
      DependencyProperty.RegisterAttached("Top", typeof(double),
      typeof(SimpleCanvas), new PropertyMetadata(0.0, OnPropertyChanged));

    static void OnPropertyChanged(DependencyObject d, 
                                  DependencyPropertyChangedEventArgs e)
    {
      FrameworkElement element = d as FrameworkElement;
      if (element != null)
      {
        SimpleCanvas canvas = element.Parent as SimpleCanvas;
        if (canvas != null)
        {
          // We don't need to remeasure, but we need to rearrange
          canvas.InvalidateArrange();
        }
      }
    }

    public static double GetLeft(UIElement element)
    {
      if (element == null) { throw new ArgumentNullException("element"); }
      return (double)element.GetValue(LeftProperty);
    }

    public static void SetLeft(UIElement element, double length)
    {
      if (element == null) { throw new ArgumentNullException("element"); }
      element.SetValue(LeftProperty, length);
    }

    public static double GetTop(UIElement element)
    {
      if (element == null) { throw new ArgumentNullException("element"); }
      return (double)element.GetValue(TopProperty);
    }

    public static void SetTop(UIElement element, double length)
    {
      if (element == null) { throw new ArgumentNullException("element"); }
      element.SetValue(TopProperty, length);
    }

    protected override Size MeasureOverride(Size availableSize)
    {
      foreach (UIElement child in this.Children)
      {
        // Give each child all the space it wants
        if (child != null)
          child.Measure(new Size(Double.PositiveInfinity,
                                 Double.PositiveInfinity));
      }

      // The SimpleCanvas itself needs no space
      return new Size(0, 0);
    }

    protected override Size ArrangeOverride(Size finalSize)
    {
      foreach (UIElement child in this.Children)
      {
        if (child != null)
        {
          // Respect any Left and Top attached properties,
          // otherwise the child is placed at (0,0)
          double x = GetLeft(child);
          double y = GetTop(child);

          // Place at the chosen (x,y) location with the child's DesiredSize
          child.Arrange(new Rect(new Point(x, y), child.DesiredSize));
        }
      }

      // Whatever size you gave me is fine
      return finalSize;
    }
  }
}